@@ -102,6 +102,9 @@ def get_args_parser():
     parser.add_argument(
         '--dev', dest='dev', choices=KNOWN_BOARDS,
         help='Generate an SD card or image for the given board.')
+    parser.add_argument(
+        '--part-table', default='mbr', choices=['mbr', 'gpt'],
+        help='Type of partition table to use for the MMC image')
     parser.add_argument(
         '--rootfs', default='ext4', choices=['ext2', 'ext3', 'ext4', 'btrfs'],
         help='Type of filesystem to use for the rootfs')
@@ -174,6 +177,11 @@ def get_args_parser():
         help="Select a bootloader from a hardware pack that contains more "
              "than one. If not specified, it will default to '%s'." %
              DEFAULT_BOOTLOADER)
+    parser.add_argument(
+        '--dtb-file',
+        help="Select a DTB file from a hardware pack that contains more "
+             "than one. If not specified, it will default to the first "
+             "entry in 'dtb_files' list.")
 
     add_common_options(parser)
     return parser
@@ -205,12 +213,29 @@ def get_android_args_parser():
         '--console', action='append', dest='consoles', default=[],
         help=('Add a console to kernel boot parameter; this parameter can be '
               'defined multiple times.'))
-    parser.add_argument(
-        '--system', default='system.tar.bz2', required=True,
-        help=('The tarball containing the Android system paritition'))
-    parser.add_argument(
-        '--userdata', default='userdata.tar.bz2', required=True,
-        help=('The tarball containing the Android data paritition'))
+
+    #group for system partition content specification
+    group = parser.add_mutually_exclusive_group(required=True)
+    group.add_argument(
+        '--system', dest="system",
+        help=('The tarball containing the Android system paritition.'
+              'Like system.tar.bz2'))
+    group.add_argument(
+        '--systemimage', dest="systemimage",
+        help=('The ext4 filesystem data file containing the Android '
+              'system paritition. Like system.img'))
+
+    #group for userdata partition content specification
+    group = parser.add_mutually_exclusive_group(required=True)
+    group.add_argument(
+        '--userdata', dest="userdata",
+        help=('The tarball containing the Android data paritition.'
+              'Like userdata.tar.bz2'))
+    group.add_argument(
+        '--userdataimage', dest="userdataimage",
+        help=('The ext4 filesystem data containing the Android '
+              'data paritition. Like userdata.img'))
+
     parser.add_argument(
         '--boot', default='boot.tar.bz2', required=True,
         help=('The tarball containing the Android root partition'))
