@@ -135,6 +135,7 @@ def copy_drop(src, dest_dir):
     final = os.path.join(dest_dir, os.path.basename(src))
     cmd = ["chown", "%s:%s" % (os.getuid(), os.getgid()), final]
     cmd_runner.run(cmd, as_root=True).wait()
+    os.chmod(final, 0644)
 
 
 class BoardException(Exception):
@@ -257,7 +258,8 @@ class BoardConfig(object):
         data, _ = self.hardwarepack_handler.get_field(field_name)
         return data
 
-    def set_metadata(self, hwpacks, bootloader=None, board=None):
+    def set_metadata(self, hwpacks, bootloader=None, board=None,
+                     dtb_file=None):
         self.hardwarepack_handler = HardwarepackHandler(hwpacks, bootloader,
                                                         board)
         with self.hardwarepack_handler:
@@ -310,6 +312,12 @@ class BoardConfig(object):
                 logger.warn("Deprecation warning: use the 'dtb_files' field "
                             "instead of 'dtb_file'.")
             self.dtb_files = self.get_metadata_field(DTB_FILES_FIELD)
+            if dtb_file:
+                dtb_dict = self._find_dtb_dict(dtb_file)
+                if dtb_dict:
+                    self.dtb_files = []
+                    self.dtb_files.append(dtb_dict)
+
             self.extra_boot_args_options = self.get_metadata_field(
                 EXTRA_BOOT_OPTIONS_FIELD)
             self.boot_script = self.get_metadata_field(BOOT_SCRIPT_FIELD)
@@ -445,15 +453,7 @@ class BoardConfig(object):
         # kill off hwpacks V1.
         return self.get_normal_sfdisk_cmd(should_align_boot_part)
 
-    def get_normal_sfdisk_cmd(self, should_align_boot_part=False):
-        """Return the sfdisk command to partition the media.
-
-        :param should_align_boot_part: Whether to align the boot partition too.
-
-        This returns a boot partition of type FAT16 or FAT32 or Linux,
-        followed by a root partition.
-        """
-
+    def get_normal_params(self, should_align_boot_part=False):
         if self.bootfs_type == 'vfat':
             if self.fat_size == 32:
                 partition_type = '0x0C'
@@ -485,6 +485,31 @@ class BoardConfig(object):
         root_start, _root_end, _root_len = align_partition(
             boot_end + 1, self.ROOT_MIN_SIZE_S, PART_ALIGN_S, PART_ALIGN_S)
 
+        return (boot_start, boot_len, partition_type, root_start)
+
+    def get_reserved_params(self, should_align_boot_part=None):
+        loader_start, loader_end, loader_len = align_partition(
+            self.loader_start_s, self.LOADER_MIN_SIZE_S, 1, PART_ALIGN_S)
+
+        boot_start, boot_end, boot_len = align_partition(
+            loader_end + 1, self.BOOT_MIN_SIZE_S, PART_ALIGN_S, PART_ALIGN_S)
+
+        root_start, _root_end, _root_len = align_partition(
+            boot_end + 1, self.ROOT_MIN_SIZE_S, PART_ALIGN_S, PART_ALIGN_S)
+
+        return (loader_start, loader_len, boot_start, boot_len, root_start)
+
+    def get_normal_sfdisk_cmd(self, should_align_boot_part=False):
+        """Return the sfdisk command to partition the media.
+
+        :param should_align_boot_part: Whether to align the boot partition too.
+
+        This returns a boot partition of type FAT16 or FAT32 or Linux,
+        followed by a root partition.
+        """
+        (boot_start, boot_len, partition_type,
+            root_start) = self.get_normal_params(should_align_boot_part)
+
         return '%s,%s,%s,*\n%s,,,-' % (
             boot_start, boot_len, partition_type, root_start)
 
@@ -496,18 +521,31 @@ class BoardConfig(object):
         This returns a loader partition, then a boot vfat partition of type
         FAT16 or FAT32, followed by a root partition.
         """
-        loader_start, loader_end, loader_len = align_partition(
-            self.loader_start_s, self.LOADER_MIN_SIZE_S, 1, PART_ALIGN_S)
-
-        boot_start, boot_end, boot_len = align_partition(
-            loader_end + 1, self.BOOT_MIN_SIZE_S, PART_ALIGN_S, PART_ALIGN_S)
-
-        root_start, _root_end, _root_len = align_partition(
-            boot_end + 1, self.ROOT_MIN_SIZE_S, PART_ALIGN_S, PART_ALIGN_S)
+        (loader_start, loader_len, boot_start, boot_len,
+            root_start) = self.get_reserved_params(should_align_boot_part)
 
         return '%s,%s,0xDA\n%s,%s,0x0C,*\n%s,,,-' % (
             loader_start, loader_len, boot_start, boot_len, root_start)
 
+    def get_normal_sgdisk_cmd(self, should_align_boot_part=None):
+        (boot_start, boot_len, partition_type,
+            root_start) = self.get_normal_params(should_align_boot_part)
+        # Ignoring partition type because we need 0xEF for GPT
+        return '-n 1:%s:%s -t 1:EF00 ' \
+               '-n 2:%s:- -t 2:8300' % (boot_start, boot_len, root_start)
+
+    def get_reserved_sgdisk_cmd(self, should_align_boot_part=None):
+        """Return the sgdisk command to partition the media.
+
+        :param should_align_boot_part: Ignored.
+        """
+        (loader_start, loader_len, boot_start, boot_len,
+            root_start) = self.get_reserved_params(should_align_boot_part)
+        return '-n 1:%s:%s -t 1:DA00 ' \
+               '-n 2:%s:%s -t 2:0C00 ' \
+               '-n 3:%s:- -t 3:8300' % (
+                   loader_start, loader_len, boot_start, boot_len, root_start)
+
     def get_sfdisk_cmd(self, should_align_boot_part=False):
         if (self.partition_layout in ['bootfs_rootfs', 'bootfs16_rootfs'] or
                 self.board == 'snowball_sd'):
@@ -520,6 +558,18 @@ class BoardConfig(object):
                 "partition_layout is unspecified.")
             return self.get_v1_sfdisk_cmd(should_align_boot_part)
 
+    def get_sgdisk_cmd(self, should_align_boot_part=False):
+        if (self.partition_layout in ['bootfs_rootfs', 'bootfs16_rootfs'] or
+                self.board == 'snowball_sd'):
+            return self.get_normal_sgdisk_cmd(should_align_boot_part)
+        elif self.partition_layout in ['reserved_bootfs_rootfs']:
+            return self.get_reserved_sgdisk_cmd(should_align_boot_part)
+        else:
+            assert (self.hwpack_format == HardwarepackHandler.FORMAT_1), (
+                "Hwpack format is not 1.0 but "
+                "partition_layout is unspecified.")
+            return self.get_normal_sgdisk_cmd(should_align_boot_part)
+
     def _get_bootcmd(self, i_img_data, d_img_data):
         """Get the bootcmd for this board.
 
@@ -577,14 +627,14 @@ class BoardConfig(object):
         boot_args_options = 'rootwait ro'
         serial_options = ''
         if self.extra_boot_args_options:
-            boot_args_options += ' %s' % self.extra_boot_args_options
+            boot_args_options += ' %s' % self.extra_boot_args_options.strip()
         if self.extra_serial_options:
-            serial_options = self.extra_serial_options
+            serial_options = self.extra_serial_options.strip()
         for console in consoles:
-            serial_options += ' console=%s' % console
+            serial_options += ' console=%s' % console.strip()
 
         lowmem_opt = ''
-        boot_snippet = 'root=%s' % rootfs_id
+        boot_snippet = 'root=%s' % rootfs_id.strip()
         if is_live:
             serial_options += ' %s' % self.live_serial_options
             boot_snippet = 'boot=casper'
@@ -873,6 +923,25 @@ class BoardConfig(object):
         if self.dtb_file:
             dtb = _get_file_matching(os.path.join(path, self.dtb_file))
         if not self.dtb_file or not dtb:
+            logger.warn("Could not find a valid dtb file from dtb_file, "
+                        "trying dtb_files...")
+
+        if self.dtb_files:
+            # Use first file from list as a default dtb file.
+            dtb_file = self.dtb_files[0]
+            if dtb_file:
+                if isinstance(dtb_file, dict):
+                    for key, value in dtb_file.iteritems():
+                        # The name of the dtb file.
+                        to_file = os.path.basename(key)
+                        from_file = value
+
+                        # User specified only the directory, without renaming
+                        # the file.
+                        if not to_file:
+                            to_file = os.path.basename(from_file)
+                        dtb = _get_file_matching(os.path.join(path, from_file))
+        if not self.dtb_files or not dtb:
             logger.warn("Could not find a valid dtb file, skipping it.")
 
         logger.info("Will use kernel=%s, initrd=%s, dtb=%s." %
@@ -900,6 +969,16 @@ class BoardConfig(object):
             presence = True
         return presence
 
+    def _find_dtb_dict(self, dtb):
+        """Returns dictionary entry from dt_files containing dtb file."""
+        for dtb_file in self.dtb_files:
+            if isinstance(dtb_file, dict):
+                for key, value in dtb_file.iteritems():
+                    # The name of the dtb file.
+                    if dtb in key:
+                        return dtb_file
+        return None
+
 
 class OmapConfig(BoardConfig):
     def __init__(self):
@@ -1413,18 +1492,36 @@ class VexpressA9Config(VexpressConfig):
         super(VexpressA9Config, self).__init__()
 
 
-class FastModelConfig(BoardConfig):
+class GenericConfig(BoardConfig):
     def __init__(self):
-        super(FastModelConfig, self).__init__()
-        self.supports_writing_to_mmc = False
+        super(GenericConfig, self).__init__()
 
     def _get_bootcmd(self, i_img_data, d_img_data):
-        """Get the bootcmd for FastModel.
+        """Get the bootcmd.
 
         We override this as we don't do uboot.
         """
         return ""
 
+    def _make_boot_files_v2(self, boot_env, chroot_dir, boot_dir,
+                            boot_device_or_file, k_img_data, i_img_data,
+                            d_img_data):
+        # Rename the kernel image
+        if k_img_data is not None:
+            k_img = os.path.join(boot_dir,
+                                 os.path.basename(k_img_data).split('-')[0])
+            cmd_runner.run(["cp", "-v", k_img_data, k_img],
+                           as_root=True).wait()
+        if i_img_data is not None:
+            cmd_runner.run(["cp", "-v", i_img_data, boot_dir],
+                           as_root=True).wait()
+
+
+class FastModelConfig(GenericConfig):
+    def __init__(self):
+        super(FastModelConfig, self).__init__()
+        self.supports_writing_to_mmc = False
+
     def _make_boot_files_v2(self, boot_env, chroot_dir, boot_dir,
                             boot_device_or_file, k_img_data, i_img_data,
                             d_img_data):
@@ -1437,22 +1534,24 @@ class FastModelConfig(BoardConfig):
 
         files = [bw_ve, bw_foundation, k_img_data, i_img_data, d_img_data]
         files.extend(glob.glob("%s/fvp/*" % boot_dir))
+        files.extend(glob.glob("%s/juno/*" % boot_dir))
+        files.extend(glob.glob("%s/boot/initrd.img-*" % chroot_dir))
         for filename in files:
             if filename is not None:
                 copy_drop(filename, output_dir)
                 cmd_runner.run(["cp", "-v", filename, boot_dir],
                                as_root=True).wait()
-        # Rename the kernel from Image-* to Image
+        # Rename the kernel image
         if k_img_data is not None:
+            k_img_basename = os.path.basename(k_img_data)
+            k_img = k_img_basename.split('-')[0]
             cmd_runner.run(["mv", "-v",
-                           os.path.join(output_dir,
-                                        os.path.basename(k_img_data)),
-                           os.path.join(output_dir, "Image")],
+                           os.path.join(output_dir, k_img_basename),
+                           os.path.join(output_dir, k_img)],
                            as_root=True).wait()
             cmd_runner.run(["mv", "-v",
-                           os.path.join(boot_dir,
-                                        os.path.basename(k_img_data)),
-                           os.path.join(boot_dir, "Image")],
+                           os.path.join(boot_dir, k_img_basename),
+                           os.path.join(boot_dir, k_img)],
                            as_root=True).wait()
 
 
@@ -1724,6 +1823,95 @@ class ArndaleConfig(SamsungConfig):
         return bl0_file
 
 
+class ArndaleOctaConfig(ArndaleConfig):
+    def __init__(self):
+        super(ArndaleOctaConfig, self).__init__()
+        self.bl0_file = 'lib/firmware/arndale-octa/arndale-octa.bl1.bin'
+        self.tzsw_file = 'lib/firmware/arndale-octa/arndale-octa.tzsw.bin'
+        self.boot_script = 'boot.scr'
+        self.bootloader_flavor = 'arndale_octa'
+        self.kernel_flavors = ['arndale-octa']
+        self.mmc_option = '0:2'
+        self.mmc_part_offset = 1
+        self.samsung_bl0_start = 1
+        self.samsung_bl0_len = 16
+        self.samsung_tzsw_start = 719
+        self.samsung_tzsw_len = 256
+
+    def _get_boot_env(self, is_live, is_lowmem, consoles, rootfs_id,
+                      i_img_data, d_img_data):
+        boot_env = super(SamsungConfig, self)._get_boot_env(
+            is_live, is_lowmem, consoles, rootfs_id, i_img_data, d_img_data)
+
+        boot_env["bootcmd"] = 'run addmac; %s' % \
+                              self._get_bootcmd(i_img_data, d_img_data)
+        boot_env["addmac"] = 'setenv bootargs "${bootargs} mac=${ethaddr}"'
+
+        return boot_env
+
+    def _make_boot_files_v2(self, boot_env, chroot_dir, boot_dir,
+                            boot_device_or_file, k_img_data, i_img_data,
+                            d_img_data):
+        with self.hardwarepack_handler:
+            bl0_file = self._get_samsung_bl0(chroot_dir)
+            if self.samsung_bl0_start:
+                self._dd_file(bl0_file, boot_device_or_file,
+                              self.samsung_bl0_start)
+
+            spl_file = self.get_file('spl_file')
+            if self.spl_in_boot_part:
+                assert spl_file is not None, (
+                    "SPL binary could not be found")
+                logger.info(
+                    "Copying spl '%s' to boot partition." % spl_file)
+                cmd_runner.run(["cp", "-v", spl_file, boot_dir],
+                               as_root=True).wait()
+                # XXX: Is this really needed?
+                cmd_runner.run(["sync"]).wait()
+
+            if self.spl_dd:
+                self._dd_file(spl_file, boot_device_or_file, self.spl_dd)
+
+            bootloader_file = self.get_file('bootloader_file')
+            if self.bootloader_dd:
+                self._dd_file(bootloader_file, boot_device_or_file,
+                              self.bootloader_dd)
+
+            tzsw_file = self._get_samsung_tzsw(chroot_dir)
+            if self.samsung_tzsw_start:
+                self._dd_file(tzsw_file, boot_device_or_file,
+                              self.samsung_tzsw_start)
+
+        make_uImage(self.load_addr, k_img_data, boot_dir)
+
+        if i_img_data is not None:
+            make_uInitrd(i_img_data, boot_dir)
+
+        if d_img_data is not None:
+            make_dtb(d_img_data, boot_dir)
+
+        if self.boot_script is not None:
+            boot_script_path = os.path.join(boot_dir, self.boot_script)
+            make_boot_script(boot_env, boot_script_path)
+
+            # Only used for Omap, will this be bad for the other boards?
+            make_boot_ini(boot_script_path, boot_dir)
+
+        if self.env_dd:
+            # Do we need to zero out the env before flashing it?
+            _dd("/dev/zero", boot_device_or_file,
+                count=self.samsung_env_len,
+                seek=self.samsung_env_start)
+            env_size = self.samsung_env_len * SECTOR_SIZE
+            env_file = make_flashable_env(boot_env, env_size)
+            self._dd_file(env_file, boot_device_or_file,
+                          self.samsung_env_start)
+
+    def _get_samsung_tzsw(self, chroot_dir):
+        tzsw_file = os.path.join(chroot_dir, self.tzsw_file)
+        return tzsw_file
+
+
 class HighBankConfig(BoardConfig):
     def __init__(self):
         super(HighBankConfig, self).__init__()
@@ -1822,14 +2010,17 @@ class BoardConfigException(Exception):
 board_configs = {
     'aa9': Aa9Config,
     'arndale': ArndaleConfig,
+    'arndale-octa': ArndaleOctaConfig,
     'beagle': BeagleConfig,
     'beaglebone': BeagleBoneConfig,
     'efikamx': EfikamxConfig,
     'efikasb': EfikasbConfig,
     'fastmodel': FastModelConfig,
+    'generic': GenericConfig,
     'highbank': HighBankConfig,
     'i386': I386Config,
     'igep': IgepConfig,
+    'juno': GenericConfig,
     'mx51evk': Mx51evkConfig,
     'mx53loco': Mx53LoCoConfig,
     'mx6qsabrelite': BoardConfig,
@@ -1937,7 +2128,7 @@ def get_plain_boot_script_contents(boot_env):
         'setenv fdt_high "%(fdt_high)s"\n'
         'setenv bootcmd "%(bootcmd)s"\n'
         'setenv bootargs "%(bootargs)s"\n'
-        "boot"
+        'boot'
         % boot_env)
 
 
